@@ -8,19 +8,19 @@ npy_double kolmogp(npy_double);
 npy_double lanczos_sum_expg_scaled(npy_double);
 npy_double lgam1p(npy_double);
 npy_double log1pmx(npy_double);
+npy_double riemann_zeta(npy_double);
 npy_double smirnovc(npy_int, npy_double);
 npy_double smirnovci(npy_int, npy_double);
 npy_double smirnovp(npy_int, npy_double);
-#include "misc.h"
 npy_double struve_asymp_large_z(npy_double, npy_double, npy_int, npy_double *);
 npy_double struve_bessel_series(npy_double, npy_double, npy_int, npy_double *);
 npy_double struve_power_series(npy_double, npy_double, npy_int, npy_double *);
 npy_double zeta(npy_double, npy_double);
 #include "amos_wrappers.h"
-npy_int cairy_wrap(npy_cdouble, npy_cdouble *, npy_cdouble *, npy_cdouble *, npy_cdouble *);
 npy_int airy_wrap(npy_double, npy_double *, npy_double *, npy_double *, npy_double *);
-npy_int cairy_wrap_e_real(npy_double, npy_double *, npy_double *, npy_double *, npy_double *);
+npy_int cairy_wrap(npy_cdouble, npy_cdouble *, npy_cdouble *, npy_cdouble *, npy_cdouble *);
 npy_int cairy_wrap_e(npy_cdouble, npy_cdouble *, npy_cdouble *, npy_cdouble *, npy_cdouble *);
+npy_int cairy_wrap_e_real(npy_double, npy_double *, npy_double *, npy_double *, npy_double *);
 npy_double bdtr(npy_int, npy_int, npy_double);
 npy_double bdtrc(npy_int, npy_int, npy_double);
 npy_double bdtri(npy_int, npy_int, npy_double);
@@ -32,7 +32,6 @@ npy_double bei_wrap(npy_double);
 npy_double beip_wrap(npy_double);
 npy_double ber_wrap(npy_double);
 npy_double berp_wrap(npy_double);
-#include "c_misc/misc.h"
 npy_double besselpoly(npy_double, npy_double, npy_double);
 npy_double beta(npy_double, npy_double);
 npy_double incbet(npy_double, npy_double, npy_double);
@@ -60,16 +59,16 @@ npy_double ellik(npy_double, npy_double);
 npy_double ellpk(npy_double);
 npy_double erf(npy_double);
 npy_double erfc(npy_double);
-npy_double exp1_wrap(npy_double);
 npy_cdouble cexp1_wrap(npy_cdouble);
+npy_double exp1_wrap(npy_double);
 npy_double exp10(npy_double);
 npy_double exp2(npy_double);
-npy_double expi_wrap(npy_double);
 npy_cdouble cexpi_wrap(npy_cdouble);
+npy_double expi_wrap(npy_double);
 #include "_logit.h"
-npy_longdouble expitl(npy_longdouble);
-npy_float expitf(npy_float);
 npy_double expit(npy_double);
+npy_float expitf(npy_float);
+npy_longdouble expitl(npy_longdouble);
 npy_double expm1(npy_double);
 npy_double expn(npy_int, npy_double);
 npy_double fdtr(npy_double, npy_double, npy_double);
@@ -94,14 +93,9 @@ npy_cdouble cbesh_wrap1(npy_double, npy_cdouble);
 npy_cdouble cbesh_wrap1_e(npy_double, npy_cdouble);
 npy_cdouble cbesh_wrap2(npy_double, npy_cdouble);
 npy_cdouble cbesh_wrap2_e(npy_double, npy_cdouble);
-npy_double hyp1f1_wrap(npy_double, npy_double, npy_double);
 npy_cdouble chyp1f1_wrap(npy_double, npy_double, npy_cdouble);
-npy_double onef2(npy_double, npy_double, npy_double, npy_double, npy_double *);
-npy_double hyp2f0(npy_double, npy_double, npy_double, npy_int, npy_double *);
 npy_double hyp2f1(npy_double, npy_double, npy_double, npy_double);
 npy_cdouble chyp2f1_wrap(npy_double, npy_double, npy_double, npy_cdouble);
-npy_double threef0(npy_double, npy_double, npy_double, npy_double, npy_double *);
-npy_double hypU_wrap(npy_double, npy_double, npy_double);
 npy_double i0(npy_double);
 npy_double i0e(npy_double);
 npy_double i1(npy_double);
@@ -116,14 +110,14 @@ npy_double itmodstruve0_wrap(npy_double);
 npy_double itstruve0_wrap(npy_double);
 npy_cdouble cbesi_wrap(npy_double, npy_cdouble);
 npy_double iv(npy_double, npy_double);
-npy_double cbesi_wrap_e_real(npy_double, npy_double);
 npy_cdouble cbesi_wrap_e(npy_double, npy_cdouble);
+npy_double cbesi_wrap_e_real(npy_double, npy_double);
 npy_double j0(npy_double);
 npy_double j1(npy_double);
 npy_cdouble cbesj_wrap(npy_double, npy_cdouble);
 npy_double cbesj_wrap_real(npy_double, npy_double);
-npy_double cbesj_wrap_e_real(npy_double, npy_double);
 npy_cdouble cbesj_wrap_e(npy_double, npy_cdouble);
+npy_double cbesj_wrap_e_real(npy_double, npy_double);
 npy_double k0(npy_double);
 npy_double k0e(npy_double);
 npy_double k1(npy_double);
@@ -136,15 +130,15 @@ npy_double kerp_wrap(npy_double);
 npy_double cbesk_wrap_real_int(npy_int, npy_double);
 npy_double kolmogi(npy_double);
 npy_double kolmogorov(npy_double);
-npy_double cbesk_wrap_real(npy_double, npy_double);
 npy_cdouble cbesk_wrap(npy_double, npy_cdouble);
+npy_double cbesk_wrap_real(npy_double, npy_double);
 npy_cdouble cbesk_wrap_e(npy_double, npy_cdouble);
 npy_double cbesk_wrap_e_real(npy_double, npy_double);
 npy_double log1p(npy_double);
 npy_double log_ndtr(npy_double);
 npy_double logit(npy_double);
-npy_longdouble logitl(npy_longdouble);
 npy_float logitf(npy_float);
+npy_longdouble logitl(npy_longdouble);
 npy_double pmv_wrap(npy_double, npy_double, npy_double);
 npy_double cem_cva_wrap(npy_double, npy_double);
 npy_double sem_cva_wrap(npy_double, npy_double);
@@ -186,8 +180,8 @@ npy_double owens_t(npy_double, npy_double);
 npy_int pbdv_wrap(npy_double, npy_double, npy_double *, npy_double *);
 npy_int pbvv_wrap(npy_double, npy_double, npy_double *, npy_double *);
 npy_int pbwa_wrap(npy_double, npy_double, npy_double *, npy_double *);
-npy_double pdtr(npy_int, npy_double);
-npy_double pdtrc(npy_int, npy_double);
+npy_double pdtr(npy_double, npy_double);
+npy_double pdtrc(npy_double, npy_double);
 npy_double pdtri(npy_int, npy_double);
 npy_double cdfpoi2_wrap(npy_double, npy_double);
 npy_double poch(npy_double, npy_double);
